<?php
declare(strict_types=1);

namespace Catequil\ORM;

use Doctrine\Common\EventManager;
use Doctrine\ORM\Configuration;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\Events;
use Doctrine\ORM\Tools\ResolveTargetEntityListener;
use Doctrine\Persistence\Mapping\Driver\MappingDriver;
use Symfony\Component\Console\Application;

/**
 * Class ConfigProvider
 *
 * @package Catequil\ORM
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-orm
 */
final class ConfigProvider
{
    public function __invoke(): array
    {
        return [
            'dependencies' => $this->getDependencies(),
            'filters' => $this->getFilters(),
            'orm' => $this->getOrmConfig(),
        ];
    }

    public function getOrmConfig(): array
    {
        return [
            'connection' => 'db.default',
            'proxy_dir' => 'data/orm/proxy',
            'annotation_namespaces' => [
                // __NAMESPACE__,
            ],
            'implementations' => [
                // __INTERFACE__ => __CLASS__
            ],
            'entity_paths' => [
                // __NAMESPACE__ => __DIR__
            ],
            'event_listeners' => [
                Events::loadClassMetadata => [
                    ResolveTargetEntityListener::class,
                ],
            ],
            'event_subscribers' => [
                // __CLASS__,
            ],
        ];
    }

    public function getDependencies(): array
    {
        return [
            'aliases' => [
                EntityManagerInterface::class => EntityManager::class,
            ],
            'delegators' => [
                Application::class => [
                    Container\CliApplicationDelegatorFactory::class,
                ],
                EventManager::class => [
                    Container\EventManagerDelegatorFactory::class,
                ],
            ],
            'factories' => [
                'orm.cache' => Container\OrmCacheFactory::class,
                Configuration::class => Container\ConfigurationFactory::class,
                EntityManager::class => Container\EntityManagerFactory::class,
                MappingDriver::class => Container\MappingDriverFactory::class,
                ResolveTargetEntityListener::class => Container\ResolveTargetEntityListenerFactory::class,
                Repository\RepositoryFactory::class => Repository\RepositoryFactoryFactory::class,
            ],
        ];
    }

    public function getFilters(): array
    {
        return [
            'factories' => [
                Filter\EntityFilter::class => Filter\EntityFilterFactory::class,
            ],
        ];
    }
}
