<?php
declare(strict_types=1);

namespace Catequil\ORM\Container;

use Catequil\ORM\Repository\RepositoryFactory;
use Doctrine\ORM\Configuration;
use Doctrine\Persistence\Mapping\Driver\MappingDriver;
use Laminas\ServiceManager\Factory\FactoryInterface;
use Psr\Cache\CacheItemPoolInterface;
use Psr\Container\ContainerExceptionInterface;
use Psr\Container\ContainerInterface;
use Psr\Container\NotFoundExceptionInterface;

/**
 * Class ConfigurationFactory
 *
 * @package Catequil\ORM\Container
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequli-orm
 */
final class ConfigurationFactory implements FactoryInterface
{
    /**
     * @param ContainerInterface $container
     * @param $requestedName
     * @param array|null $options
     *
     * @return Configuration
     *
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function __invoke(ContainerInterface $container, $requestedName, ?array $options = null): Configuration
    {
        $config = $container->has('config') ? $container->get('config') : [];
        $isDevelopment = $config['development'] ?? ($config['debug'] ?? false);
        $config = $config['orm'] ?? [];

        /** @var CacheItemPoolInterface $cache */
        $cache = $container->get('orm.cache');
        /** @var MappingDriver $driver */
        $driver = $container->get(MappingDriver::class);
        /** @var RepositoryFactory $repositoryFactory */
        $repositoryFactory = $container->get(RepositoryFactory::class);

        $configuration = new Configuration();
        $configuration->setAutoGenerateProxyClasses($isDevelopment);
        $configuration->setMetadataCache($cache);
        $configuration->setMetadataDriverImpl($driver);
        $configuration->setQueryCache($cache);
        $configuration->setRepositoryFactory($repositoryFactory);
        $configuration->setResultCache($cache);
        $configuration->setProxyDir($config['proxy_dir'] ?? null);
        $configuration->setProxyNamespace('Catequil\\ORM\\Proxy\\');

        return $configuration;
    }
}
