<?php
declare(strict_types=1);

namespace Catequil\ORM\Container;

use Catequil\Db\Connection;
use Doctrine\ORM\Configuration;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\Exception\ORMException;
use Laminas\ServiceManager\Exception\ServiceNotCreatedException;
use Laminas\ServiceManager\Factory\FactoryInterface;
use Psr\Container\ContainerExceptionInterface;
use Psr\Container\ContainerInterface;
use Psr\Container\NotFoundExceptionInterface;

/**
 * Class EntityManagerFactory
 *
 * @package Catequil\ORM\Container
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-orm
 */
final class EntityManagerFactory implements FactoryInterface
{
    /**
     * @param ContainerInterface $container
     * @param $requestedName
     * @param array|null $options
     *
     * @return EntityManagerInterface
     *
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function __invoke(ContainerInterface $container, $requestedName, ?array $options = null): EntityManagerInterface
    {
        $config = $container->get('config')['orm'] ?? [];

        /** @var Connection $connection */
        $connection = $options['connection'] ?? $container->get($config['connection'] ?? 'db');
        /** @var Configuration $configuration */
        $configuration = $container->get(Configuration::class);
        $events = $connection->getEventManager();

        try {
            return EntityManager::create($connection, $configuration, $events);
        } catch (ORMException $e) {
            throw new ServiceNotCreatedException('Can\'t create the entity manager.', 0, $e);
        }
    }
}
