<?php
declare(strict_types=1);

namespace Catequil\ORM\Container;

use Doctrine\Common\EventManager;
use Laminas\ServiceManager\Exception\ServiceNotCreatedException;
use Laminas\ServiceManager\Exception\ServiceNotFoundException;
use Laminas\ServiceManager\Factory\DelegatorFactoryInterface;
use Psr\Container\ContainerExceptionInterface;
use Psr\Container\ContainerInterface;
use Psr\Container\NotFoundExceptionInterface;
use function is_string;

/**
 * Class EventManagerDelegatorFactory
 *
 * @package Catequil\ORM\Container
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-orm
 */
final class EventManagerDelegatorFactory implements DelegatorFactoryInterface
{
    /**
     * @param ContainerInterface $container
     * @param $name
     * @param callable $callback
     * @param array|null $options
     *
     * @return EventManager
     *
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function __invoke(ContainerInterface $container, $name, callable $callback, ?array $options = null): EventManager
    {
        /** @var EventManager $events */
        $events = $callback();
        $config = $container->get('config')['orm'] ?? [];

        foreach ($config['event_listeners'] ?? [] as $event => $listeners) {
            foreach ($listeners as $listener) {
                if (is_string($listener)) {
                    $listener = $container->get($listener);
                }
                $events->addEventListener($event, $listener);
            }
        }

        foreach ($config['event_subscribers'] ?? [] as $subscriber) {
            if (is_string($subscriber)) {
                $subscriber = $container->get($subscriber);
            }
            $events->addEventSubscriber($subscriber);
        }

        return $events;
    }
}