<?php
declare(strict_types=1);

namespace Catequil\ORM\Container;

use Doctrine\Common\Annotations\AnnotationReader;
use Doctrine\Common\Annotations\PsrCachedReader;
use Doctrine\ORM\Mapping\Driver\AnnotationDriver;
use Doctrine\ORM\Mapping\Driver\AttributeDriver;
use Doctrine\Persistence\Mapping\Driver\MappingDriver;
use Doctrine\Persistence\Mapping\Driver\MappingDriverChain;
use Laminas\ServiceManager\Factory\FactoryInterface;
use Psr\Container\ContainerExceptionInterface;
use Psr\Container\ContainerInterface;
use Psr\Container\NotFoundExceptionInterface;
use function in_array;

/**
 * Class MappingDriverFactory
 *
 * @package Catequil\ORM\Container
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-orm
 */
final class MappingDriverFactory implements FactoryInterface
{
    /**
     * @param ContainerInterface $container
     * @param $requestedName
     * @param array|null $options
     *
     * @return MappingDriver
     *
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function __invoke(ContainerInterface $container, $requestedName, ?array $options = null): MappingDriver
    {
        $config = $container->has('config') ? $container->get('config') : [];
        $isDevelopment = $config['development'] ?? ($config['debug'] ?? false);
        $config = $config['orm'] ?? [];

        $cache = $container->get('orm.cache');
        $reader = new PsrCachedReader(new AnnotationReader(), $cache, $isDevelopment);

        $chain = new MappingDriverChain();

        foreach ($config['entity_paths'] ?? [] as $namespace => $paths) {
            $driver = in_array($namespace, $config['annotation_namespaces'])
                ? new AnnotationDriver($reader, (array) $paths)
                : new AttributeDriver((array) $paths);

            $chain->addDriver($driver, $namespace);
        }

        return $chain;
    }
}
