<?php
declare(strict_types=1);

namespace Catequil\ORM\Container;

use Doctrine\ORM\Tools\ResolveTargetEntityListener;
use Laminas\ServiceManager\Exception\ServiceNotCreatedException;
use Laminas\ServiceManager\Exception\ServiceNotFoundException;
use Laminas\ServiceManager\Factory\FactoryInterface;
use Psr\Container\ContainerExceptionInterface;
use Psr\Container\ContainerInterface;
use Psr\Container\NotFoundExceptionInterface;

/**
 * Class ResolveTargetEntityListenerFactory
 *
 * @package Catequil\ORM\Container
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-orm
 */
final class ResolveTargetEntityListenerFactory implements FactoryInterface
{
    /**
     * @param ContainerInterface $container
     * @param $requestedName
     * @param array|null $options
     *
     * @return ResolveTargetEntityListener
     *
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function __invoke(ContainerInterface $container, $requestedName, ?array $options = null): ResolveTargetEntityListener
    {
        $config = $container->get('config')['orm'] ?? [];
        $rtel = new ResolveTargetEntityListener();

        foreach ($config['implementations'] ?? [] as $interface => $implementation) {
            $rtel->addResolveTargetEntity($interface, $implementation, []);
        }

        return $rtel;
    }
}