<?php
declare(strict_types=1);

namespace CatequilTest\ORM;

use Catequil\ORM\Repository\RepositoryPluginManager;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\EntityManagerInterface;
use PHPUnit\Framework\TestCase;
use Psr\Container\ContainerInterface;
use function array_merge;

/**
 * Class AbstractTestCase
 *
 * @package CatequilTest\ORM
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-orm
 */
abstract class AbstractTestCase extends TestCase
{
    /**
     * @param array $config
     *
     * @return ContainerInterface
     */
    protected static function getContainer(array $config = []): ContainerInterface
    {
        $config = array_merge([
            'orm' => [
                'connection' => 'db.memory',
                'entity_paths' => [
                    __NAMESPACE__ => __DIR__ . '/Entity',
                ],
            ],
            'db' => [
                'connections' => [
                    'db.memory' => [
                        'driver' => 'pdo_sqlite',
                        'memory' => true,
                    ],
                ],
            ],
        ], $config);

        return include __DIR__ . '/container.php';
    }

    protected static function getEntityManager(array $config = []): EntityManager
    {
        $container = self::getContainer($config);
        /** @noinspection PhpUnhandledExceptionInspection */
        return $container->get(EntityManagerInterface::class);
    }
}
